<?php
defined('BASEPATH') OR exit('No direct script access allowed');
 
class mercadopago extends MX_Controller {
	public $tb_users;
	public $tb_transaction_logs;
	public $tb_payments;
	public $tb_payments_bonuses;
	public $paypal;
	public $payment_type;
	public $payment_id;
	public $currency_code;
	public $payment_lib;
	public $mode;
	
	public $public_key;
	public $access_token;
	public $currency_rate_to_usd;

	public function __construct($payment = ""){
		parent::__construct();
		$this->load->model('add_funds_model', 'model');

		$this->tb_users            = USERS;
		$this->tb_transaction_logs = TRANSACTION_LOGS;
		$this->tb_payments         = PAYMENTS_METHOD;
		$this->tb_payments_bonuses = PAYMENTS_BONUSES;
		$this->payment_type		   = get_class($this);
		$this->currency_code       = get_option("currency_code", "USD");
		if ($this->currency_code == "") {
			$this->currency_code = 'USD';
		}
		if (!$payment) {
			$payment = $this->model->get('id, type, name, params', $this->tb_payments, ['type' => $this->payment_type]);
		}
		$this->payment_id 	      = $payment->id;
		$params  			      = $payment->params;
		$option                   = get_value($params, 'option');
		$this->mode               = get_value($option, 'environment');
		$this->payment_fee        = get_value($option, 'tnx_fee');

		// Payment Option
		$this->public_key          		= get_value($option, 'public_key');
		$this->access_token       		= get_value($option, 'access_token');
		$this->currency_rate_to_usd     = get_value($option, 'rate_to_usd');
		$this->load->library("mercadopagoapi");
		$this->payment_lib = new mercadopagoapi($this->access_token);
	}

	public function index(){
		redirect(cn('add_funds'));
	}

	/*----------  Create payments  ----------*/
	public function create_payment($data_payment = ""){
		_is_ajax($data_payment['module']);
		$amount = $data_payment['amount'];
		if (!$amount) {
			_validation('error', lang('There_was_an_error_processing_your_request_Please_try_again_later'));
		}

		if (!$this->public_key || !$this->access_token) {
			_validation('error', lang('this_payment_is_not_active_please_choose_another_payment_or_contact_us_for_more_detail'));
		}

		$users = session('user_current_info');
		$description   = lang("Balance_recharge")." - ".  $users['email'];
		$amount = number_format($amount, 2, '.', ',');

		$data_payment = array(
			"amount" 	          => $amount,
			"currency_code" 	  => $this->currency_code ,
			"order_id" 	          => "ORDS" . strtotime(NOW),
			"email" 	          => $users['email'],
		    "description"         => $description,
			"backUrl"             => cn("add_funds/mercadopago/complete"),
			"ipnUrl"              => cn('mercadopago_ipn'),
			"mode"                => $this->mode,
		);

		$payment = $this->payment_lib->create_payment($data_payment);

		if (isset($payment->status) && $payment->status == 'error') {
			ms(array(
				"status"  => "error",
				"message" => $payment->message
			));
		}

		if (isset($payment->status) && isset($payment->redirect_url) && $payment->redirect_url) {
			$converted_amount = $amount / $this->currency_rate_to_usd;
			$data_tnx_log = array(
				"ids" 				=> ids(),
				"uid" 				=> session("uid"),
				"type" 				=> $this->payment_type,
				"transaction_id" 	=> $payment->preference_id,
				"amount" 	        => round($converted_amount, 4) ,
				'txn_fee'           => round($converted_amount * ($this->payment_fee / 100), 4),
				"note" 	            => $amount,
				"status" 	        => 0,
				"created" 			=> NOW,
			);
			$transaction_log_id = $this->db->insert($this->tb_transaction_logs, $data_tnx_log);
			$this->load->view('redirect', ['redirect_url' => $payment->redirect_url]);
		}else{
			_validation('error', lang('There_was_an_error_processing_your_request_Please_try_again_later'));
		}
	}

	public function complete(){
		if (!isset($_REQUEST) && $_REQUEST['merchant_order_id'] != "" && $_REQUEST['collection_id'] != "" && $_REQUEST['order_id']) {
			redirect(cn('add_funds'));
		}
		$order_id = $_REQUEST['order_id'];
		$order_id = strip_tags($order_id);
		$transaction = $this->model->get('*', $this->tb_transaction_logs, ['transaction_id' => $order_id, 'status' => 0, 'type' => $this->payment_type]);

		if (!$transaction) {
			redirect(cn("add_funds"));
		}
		
		$data_payment = [
			'merchant_order_id' => $_REQUEST['merchant_order_id'],
			'collection_id'     => $_REQUEST['collection_id'],
		];
		$check_payment = $this->mercadopagoapi->get_id_details((object)$data_payment);
       
		if (isset($check_payment->status) && $check_payment->status == 'completed' && $_REQUEST['collection_status'] == 'approved' && $transaction) {

			$this->db->update($this->tb_transaction_logs, [ 'status' => 1, 'transaction_id' => $check_payment->data->id, 'data' => json_encode($_REQUEST) ],  ['id' => $transaction->id]);

            // Update Balance 
        	require_once 'add_funds.php';
        	$add_funds = new add_funds();
        	$add_funds->add_funds_bonus_email($transaction, $this->payment_id);
            	
			set_session("transaction_id", $transaction->id);
			redirect(cn("add_funds/success"));
		} else {
			$this->db->update($this->tb_transaction_logs, [ 'data' => json_encode($_REQUEST) ] ,['id' => $transaction->id, 'type' => $this->payment_type]);
			redirect(cn("add_funds"));
		}
	}

	public function cron(){
		$this->load->model('model', 'help_model');
		$transactions = $this->help_model->fetch( '*' , $this->tb_transaction_logs, ['status' => 0, 'type' => $this->payment_type]);
		if (!empty( $transactions ) ) {
			foreach ( $transactions as $key => $transaction) {
				$data_payments = json_decode($transaction->data);
				if (empty($data_payments)) {
					continue;
				}
				$data_check = [
					'merchant_order_id' => $data_payments->merchant_order_id,
					'collection_id'     => $data_payments->collection_id,
				];

				$result = $this->mercadopagoapi->get_id_details((object)$data_check);
				if (isset($result->status) && $result->status == 'completed') {

					$this->db->update($this->tb_transaction_logs, [ 'status' => 1, 'transaction_id' => $result->data->id, 'data' => json_encode($_REQUEST) ],  ['id' => $transaction->id]);

		            // Update Balance 
		        	require_once 'add_funds.php';
		        	$add_funds = new add_funds();
		        	$add_funds->add_funds_bonus_email($transaction, $this->payment_id);
				}

				if (isset($result->status) && $result->status == 'canceled') {
					$this->db->update($this->tb_transaction_logs, [ 'status' => -1],  ['id' => $transaction->id]);
				}
			}
		}else{
			echo "There is no Transaction at the present<br>";
		}
		echo "Successfully";
	}

    public function ipn(){
		$topic 			= strip_tags($_REQUEST['topic']);
		$payment_id   	= strip_tags($_REQUEST['id']);
		if(!isset($payment_id) && !$topic){
			echo "There is no Transaction at the present<br>";
			exit();
		}
		file_put_contents($_SERVER['DOCUMENT_ROOT'].'/mx_ipn.txt', json_encode($_REQUEST).PHP_EOL, FILE_APPEND);
		$result = $this->mercadopagoapi->get_payment_id_details($_REQUEST);
		file_put_contents($_SERVER['DOCUMENT_ROOT'].'/mx_ipn_result.txt', json_encode($result).PHP_EOL, FILE_APPEND);
		
		if(isset($result->preference_id)){
			$transaction = $this->model->get('*', $this->tb_transaction_logs, ['transaction_id' => $result->preference_id, 'status' => 0, 'type' => $this->payment_type]);
			if($transaction && isset($result->status) && $result->status == 'completed'){
				
				$this->db->update($this->tb_transaction_logs, [ 'status' => 1, 'data' => json_encode($result) ],  ['id' => $transaction->id]);

				// Update Balance 
				require_once 'add_funds.php';
				$add_funds = new add_funds();
				$add_funds->add_funds_bonus_email($transaction, $this->payment_id);
				echo 'Paid';
				exit();
			}else{
				echo "There is no Transaction at the present!!<br>";
				exit();
			}
		}
		
	}
}