<?php defined('BASEPATH') OR exit('No direct script access allowed');
require_once 'mercadopago/autoload.php';

/**
 * 
 */
class mercadopagoapi{
	
	public function __construct($mercadopago_access_token = null) {
	    MercadoPago\SDK::setAccessToken($mercadopago_access_token);

	}

	/**
     *
     * Define Payment && Create payment.
     *
     */
    public function create_payment($data = array()){

        try {
            $amount = $data['currency_code'];
            // Create a preference object
            $preference        = new \MercadoPago\Preference();
            // Create an item in the preference
            $item              = new \MercadoPago\Item();
            $item->title       = $data['description'];
            $item->quantity    = 1;
            $item->unit_price  = $data['amount'];
            $preference->items = array($item);
            $orderID           = $data['order_id'];
            $backUrl           = $data['backUrl'].'?paymentOption=mercadopago&order_id='.$orderID .'&amount='. $item->unit_price;
            // Back Urls
            $preference->back_urls = array(
                "success" => $backUrl,
                "failure" => $backUrl,
                "pending" => $backUrl
            );

            $ipnUrl = $data['ipnUrl'];
            $preference->notification_url = $ipnUrl;
            $preference->auto_return = "approved";
            $preference->save();

            // Create a customer object
            $payer = new \MercadoPago\Payer();
            // Create payer information
            $payer->name = $data['email'];
            $payer->email = $data['email'];
            $payer->address = array(
                "street_name" => 'Av. Brig. Faria Lima'
            );

            // Check if test mode true or false
            // and set redirect url as per
            if ($data['mode'] == 'TEST') {
                $redirectUrl = $preference->sandbox_init_point;
            } else {
                $redirectUrl = $preference->init_point;
            }
            
            return (object)[
                'status'        => 'success',
                'preference_id' => $preference->id,
                'redirect_url'  => $redirectUrl
            ];
        } catch (Exception $e) {
            return [
                'status'   => 'error',
                'message'  => $e->getMessage()
            ];
        }

    }

    public function get_id_details($data_payment = "") {
        $merchant_order = null;
        $merchant_order = \MercadoPago\MerchantOrder::find_by_id($data_payment->merchant_order_id);
        if (!isset($merchant_order)) {
            $result = [
                'status'    => 'cancelled',
            ];
            return (object)$result;
        }
       
        foreach ($merchant_order->payments as $payment) {
            if ($payment->id == $data_payment->collection_id && isset($payment->status)) {
                // completed
                if (strtolower($payment->status) == "approved") {
                    $result = [
                        'status'    => 'completed',
                        'data'      => $payment
                    ];
                    return (object)$result;
                }
                // cancelled
                if (in_array(strtolower($payment->status), ['rejected', 'cancelled', 'refunded', 'charged_back'])) {
                    $result = [
                        'status'    => 'canceled',
                        'data'      => $payment
                    ];
                    return (object)$result;
                }
                // In processing
                if (in_array(strtolower($payment->status), ['pending', 'authorized', 'in_process', 'in_mediation'])) {
                    $result = [
                        'status'    => 'pending',
                        'data'      => $payment
                    ];
                    return (object)$result;
                }

            }else{
                $result = [
                    'status'    => 'cancelled',
                ];
                return (object)$result;
            }
        }
        
    }
    
    public function get_payment_id_details($params = ""){
        $pm_topic = strip_tags($params['topic']);
        $pm_id    = strip_tags($params['id']);
        switch($pm_topic) {
            case "payment":
                $payment = MercadoPago\Payment::find_by_id($pm_id);
                // Get the payment and the corresponding merchant_order reported by the IPN.
                $merchant_order = MercadoPago\MerchantOrder::find_by_id($payment->order->id);
                break;
            case "merchant_order":
                $merchant_order = MercadoPago\MerchantOrder::find_by_id($pm_id );
                break;
        }
        
        if (!isset($merchant_order)) {
            $result = [
                'status'    => 'canceled',
            ];
        }
        file_put_contents($_SERVER['DOCUMENT_ROOT'].'/mx_ipn2.txt', json_encode($merchant_order).PHP_EOL, FILE_APPEND);
       
        $paid_amount = 0;
        foreach ($merchant_order->payments as $payment) {
            
            if ($payment->status == 'approved'){
                $paid_amount += $payment->transaction_amount;
            }
        }
        
        // If the payment's transaction amount is equal (or bigger) than the merchant_order's amount you can release your items
        if($paid_amount >= $merchant_order->total_amount){
            $result = [
                'preference_id'   => $merchant_order->preference_id,
                'status'          => 'completed',
                'data'            => $merchant_order->payments
            ];
            //   print_r("Totally paid. Release your item.");
            
        } else {
            $result = [
                'preference_id'   => $merchant_order->preference_id,
                'status'          => 'canceled',
                'data'            => $merchant_order->payments
            ];
        }
        return (object)$result;
    }
}


